/*
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * This file is part of X2C. http://x2c.lcm.at/
 * $LastChangedRevision: 1603 $
 */
// USERCODE-BEGIN:Description                                                                                           
/* Description: */
// USERCODE-END:Description                                                                                             

package at.lcm.x2c.library.general;

import at.lcm.x2c.core.structure.*;
import at.lcm.bu21.general.dtypes.*;

// USERCODE-BEGIN:Imports                                                                                               
import at.lcm.x2c.utils.QFormat;
// USERCODE-END:Imports                                                                                                 

@SuppressWarnings("unused")
public class ConvFnc_LookupTable2D_FiP32 extends JavaConversionFunction {
    private static final long serialVersionUID = 1L;

    @Override
    public void convertMaskToImplementation() throws Exception {

        // Mask Parameter: TableData                                                                                    
        MaskDouble TableDataMaskVal = 
            (MaskDouble)this.getMaskParameter("TableData").getMaskDataType();
        TNumeric TableDataMaskData = 
            (TNumeric)TableDataMaskVal.getData();
        // Mask Parameter: DimX                                                                                         
        MaskComboBox DimXMaskVal = 
            (MaskComboBox)this.getMaskParameter("DimX").getMaskDataType();
        TString DimXMaskData = 
            (TString)DimXMaskVal.getData();
        // Mask Parameter: DimY                                                                                         
        MaskComboBox DimYMaskVal = 
            (MaskComboBox)this.getMaskParameter("DimY").getMaskDataType();
        TString DimYMaskData = 
            (TString)DimYMaskVal.getData();
        // Mask Parameter: SignalTypeX                                                                                  
        MaskComboBox SignalTypeXMaskVal = 
            (MaskComboBox)this.getMaskParameter("SignalTypeX").getMaskDataType();
        TString SignalTypeXMaskData = 
            (TString)SignalTypeXMaskVal.getData();
        // Mask Parameter: SignalTypeY                                                                                  
        MaskComboBox SignalTypeYMaskVal = 
            (MaskComboBox)this.getMaskParameter("SignalTypeY").getMaskDataType();
        TString SignalTypeYMaskData = 
            (TString)SignalTypeYMaskVal.getData();

        // Implementation Parameter: Table                                                                              
        TNumeric TableCtrVal = this.getImplementationParameterData("Table");
        // Implementation Parameter: sfrX                                                                               
        TNumeric sfrXCtrVal = this.getImplementationParameterData("sfrX");
        // Implementation Parameter: sfrY                                                                               
        TNumeric sfrYCtrVal = this.getImplementationParameterData("sfrY");

// USERCODE-BEGIN:ConvMaskToImplementation                                                                              
		int size_x = 256;
		int size_y = 256;
		final int SIZE_ROW = 1;
		final int BITS = 32;

		/* get table size */
		size_x = Integer.valueOf(DimXMaskVal.getValue());
		size_y = Integer.valueOf(DimYMaskVal.getValue());

		/* check table data for correct size */
		if ((TableDataMaskData.getNumColumns() != (size_x * size_y)) || (TableDataMaskData.getNumRows() != SIZE_ROW)) {
			throw new Exception("Lookup Table Size mismatch in column: expected=(" + SIZE_ROW + ", "
					+ (size_x * size_y) + ") " + "got=(" + TableDataMaskData.getNumRows() + ","
					+ TableDataMaskData.getNumColumns() + ")");
		}

		/* convert dimension shift factor */
		sfrXCtrVal.setReal(0, 0, Double.valueOf(QFormat.getQFormat(size_x, BITS + 2, true)));
		sfrYCtrVal.setReal(0, 0, Double.valueOf(QFormat.getQFormat(size_y, BITS + 2, true)));

		/* convert table data */
		Double[] tmpData = new Double[(size_x + 1) * (size_y + 1)];
		for (int y = 0; y < size_y; y++) {
			for (int x = 0; x < size_x; x++) {
				tmpData[x + ((size_x + 1) * y)] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, x + ((size_x) * y)), BITS - 1, BITS, true));
			}
			/* add extra x-entry to prevent index overflow during interpolation */
			if (SignalTypeXMaskVal.getValue().equals("Periodic")) {
				/* periodic signal -> add first entry at end of "row" */
				tmpData[size_x + ((size_x + 1) * y)] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, 0 + ((size_x) * y)), BITS - 1, BITS, true));
			} else {
				/* non-periodic signal -> add last entry at end of "row" */
				tmpData[size_x + ((size_x + 1) * y)] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, (size_x - 1) + ((size_x) * y)), BITS - 1, BITS, true));
			}
		}
		/* add extra y-entry to prevent index overflow during interpolation */
		if (SignalTypeYMaskVal.getValue().equals("Periodic")) {
			/* periodic signal -> add first entry at end of "column" */
			for (int y = 0; y < size_x; y++) {
				tmpData[((size_x + 1) * size_y) + y] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, 0 + y), BITS - 1, BITS, true));
			}
		} else {
			/* non-periodic signal -> add last entry at end of "column" */
			for (int y = 0; y < size_x; y++) {
				tmpData[((size_x + 1) * size_y) + y] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, (size_x * (size_y - 1)) + y), BITS - 1, BITS, true));
			}
		}
		/* add very last x,y-entry to prevent index overflow during interpolation */
		if (SignalTypeXMaskVal.getValue().equals("Periodic")) {
			if (SignalTypeYMaskVal.getValue().equals("Periodic")) {
				/* periodic signal x and y-> add first entry at end of table */
				tmpData[((size_x + 1) * (size_y + 1)) - 1] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, 0), BITS - 1, BITS, true));
			} else {
				/* periodic signal x and non-periodic signal y */
				tmpData[((size_x + 1) * (size_y + 1)) - 1] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, (size_x + 1) * (size_y - 1)), BITS - 1, BITS, true));
			}
		} else {
			if (SignalTypeYMaskVal.getValue().equals("Periodic")) {
				/* non-periodic signal x and periodic signal y */
				tmpData[((size_x + 1) * (size_y + 1)) - 1] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, size_x - 1), BITS - 1, BITS, true));
			} else {
				/* non-periodic signal x and y-> add last entry at end of table */
				tmpData[((size_x + 1) * (size_y + 1)) - 1] = Double.valueOf(QFormat.getQValue(
						TableDataMaskData.getReal(0, (size_x * size_y) - 1), BITS - 1, BITS, true));
			}
		}


		TableCtrVal = new TNumeric(tmpData);

// USERCODE-END:ConvMaskToImplementation                                                                                
        this.setImplementationParameterData("Table", TableCtrVal);
        this.setImplementationParameterData("sfrX", sfrXCtrVal);
        this.setImplementationParameterData("sfrY", sfrYCtrVal);
    }

    @Override
    public void convertImplementationToMask() throws Exception {

        // Mask Parameter: TableData                                                                                    
        MaskDouble TableDataMaskVal = 
            (MaskDouble)this.getMaskParameter("TableData").getMaskDataType();
        TNumeric TableDataMaskData = 
            (TNumeric)TableDataMaskVal.getData();
        // Mask Parameter: DimX                                                                                         
        MaskComboBox DimXMaskVal = 
            (MaskComboBox)this.getMaskParameter("DimX").getMaskDataType();
        TString DimXMaskData = 
            (TString)DimXMaskVal.getData();
        // Mask Parameter: DimY                                                                                         
        MaskComboBox DimYMaskVal = 
            (MaskComboBox)this.getMaskParameter("DimY").getMaskDataType();
        TString DimYMaskData = 
            (TString)DimYMaskVal.getData();
        // Mask Parameter: SignalTypeX                                                                                  
        MaskComboBox SignalTypeXMaskVal = 
            (MaskComboBox)this.getMaskParameter("SignalTypeX").getMaskDataType();
        TString SignalTypeXMaskData = 
            (TString)SignalTypeXMaskVal.getData();
        // Mask Parameter: SignalTypeY                                                                                  
        MaskComboBox SignalTypeYMaskVal = 
            (MaskComboBox)this.getMaskParameter("SignalTypeY").getMaskDataType();
        TString SignalTypeYMaskData = 
            (TString)SignalTypeYMaskVal.getData();

        // Implementation Parameter: Table                                                                              
        TNumeric TableCtrVal = this.getImplementationParameterData("Table");
        // Implementation Parameter: sfrX                                                                               
        TNumeric sfrXCtrVal = this.getImplementationParameterData("sfrX");
        // Implementation Parameter: sfrY                                                                               
        TNumeric sfrYCtrVal = this.getImplementationParameterData("sfrY");

// USERCODE-BEGIN:ConvImplementationToMask                                                                              
		/* TODO Add Implementation- to Mask conversion */
// USERCODE-END:ConvImplementationToMask                                                                                
        this.getMaskParameter("TableData").setValue(TableDataMaskData.toFullString());
        this.getMaskParameter("DimX").setValue(DimXMaskData.getString());
        this.getMaskParameter("DimY").setValue(DimYMaskData.getString());
        this.getMaskParameter("SignalTypeX").setValue(SignalTypeXMaskData.getString());
        this.getMaskParameter("SignalTypeY").setValue(SignalTypeYMaskData.getString());
    }
}
