/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.jni.*;

/**
 * This value class represents the VCI Version. The Version can be obtained by
 * calling {@link VciServer#GetVersion()}
 * 
 * @author Michael Ummenhofer
 * 
 */
public class VciVersion
{
  /** Major VCI server version number */
  public int m_dwVciMajorVersion  = 0;
  /** Minor VCI server version number */
  public int m_dwVciMinorVersion  = 0;  
  /** Major OS version number  */
  public int m_dwOsMajorVersion   = 0; 
  /** Minor OS version number  */
  public int m_dwOsMinorVersion   = 0;
  /** OS build number  */
  public int m_dwOsBuildNumber    = 0;
  /** OS platform id */
  public int m_dwOsPlatformId     = 0;
  
  /**
   * Creates a new VCI Version Info object 
   * @param pVciVersionInfo
   *          CPointer to native memory which contains the VCIVERSIONINFO
   *          structure.
   * @throws Throwable
   */
  protected VciVersion(CPointer pVciVersionInfo) throws Throwable
  {
    m_dwVciMajorVersion = pVciVersionInfo.getInt(0);
    m_dwVciMinorVersion = pVciVersionInfo.getInt(4);
    m_dwOsMajorVersion  = pVciVersionInfo.getInt(8);
    m_dwOsMinorVersion  = pVciVersionInfo.getInt(12);
    m_dwOsBuildNumber   = pVciVersionInfo.getInt(16);
    m_dwOsPlatformId    = pVciVersionInfo.getInt(20);
  }

  /**
   * Returns the Version as string.
   * @return The Version as string.
   */
  public String toString()
  {
    String strVersion;
    strVersion =  "VCI Version " + m_dwVciMajorVersion + "." + m_dwVciMinorVersion;
    strVersion += ", OS Version: " + m_dwOsMajorVersion + "." + m_dwOsMinorVersion + " Build " + m_dwOsBuildNumber + ", ID: " + m_dwOsPlatformId;
    
    return strVersion;
  }

}
