/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.jni.*;
import de.ixxat.vci3.bal.FifoWriter;

/**
 * This class implements the interface {@link ICanMessageWriter} which
 * represents a CAN message writer.
 * 
 * @see ICanMessageWriter
 * @author Michael Ummenhofer
 *
 */
public class CanMessageWriter extends FifoWriter implements ICanMessageWriter
{

  /**
   * constructor
   * @param pIFifoWriter
   * @throws Throwable
   */
  protected CanMessageWriter(CMalloc pIFifoWriter) throws Throwable
  {
    super(pIFifoWriter);
    //System.out.println("CanMessageWriter::CanMessageWriter");
  }

  /**
   * Clean up Object
   * @throws Throwable
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see ICanMessageWriter#WriteMessage(CanMessage)
   * @throws Throwable
   */
  public void WriteMessage(CanMessage oCanMsg) throws Throwable
  {
    CPointer pData = null;
    
    synchronized (this)
    {
      try
      {
        // Lock access to FiFo
        super.Lock();
        
        // Try to get a Pointer to a free FiFo Entry
        pData = super.AcquireWrite();
        
        // Copy Message into FiFo and release message
        if(pData != null)
        {
          // Copy Message into FiFo
          oCanMsg.toCPointer(pData);
          super.ReleaseWrite();
        }
      }
      finally
      {
        // Unlock access to FiFo
        super.Unlock();
      }
    }
    
  }

}
