/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;;

/**
 * This value class represents the LIN Line Status.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class LinLineStatus
{
  /** transmission pending */
  private static final int LIN_STATUS_OVRRUN = 0x01;
  /** init mode active */
  private static final int LIN_STATUS_ININIT = 0x10;

  /** current LIN operating mode */
  public byte   m_bOpMode   = 0;  
  /** reserved */
  public byte   m_bReserved = 0;  
  /** current bit rate */
  public short  m_wBitrate  = 0;  
  /** status of the LIN controller (see LIN_STATUS_) */
  private int   m_dwStatus  = 0; 
  
  
  /**
   * data overrun occurred
   * <p>
   * For {@link #toString()}: <code>"C"</code> means data overrun occurred
   */
  public boolean m_fRxOverrun = false;
  
  /**
   * init mode active
   * <p>
   * For {@link #toString()}: <code>"I"</code> means init mode active
   */
  public boolean m_fLinInitMode = false;
  
  
  /**
   * Creates a new LIN Line Status object.
   * 
   * @throws Throwable
   */
  protected LinLineStatus()
  {
  }
  
  /**
   * Creates a new LIN Line Status object.
   * 
   * @param pLinLineStatus
   *          CPointer to native memory which contains the LINLINESTATUS
   *          structure.
   * @throws Throwable
   */
  protected LinLineStatus(CPointer pLinLineStatus) throws Throwable
  {
    fromCPointer(pLinLineStatus);
  }
  
  /**
   * Updates a existing LIN Line Status object.
   * 
   * @param pLinLineStatus
   *          CPointer to native memory which contains the LINLINESTATUS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pLinLineStatus) throws Throwable
  {
    m_bOpMode   = pLinLineStatus.getByte(0);
    m_bReserved = pLinLineStatus.getByte(1);
    m_wBitrate  = pLinLineStatus.getShort(2);
    m_dwStatus  = pLinLineStatus.getInt(4);
    
    if((m_dwStatus & LIN_STATUS_OVRRUN) == LIN_STATUS_OVRRUN)
      m_fRxOverrun = true;
    else
      m_fRxOverrun = false;
    
    if((m_dwStatus & LIN_STATUS_ININIT) == LIN_STATUS_ININIT)
      m_fLinInitMode = true;
    else
      m_fLinInitMode = false;
  }

  /**
   * Returns the LIN Line Status as string.
   * @return The LIN Line Status as string.
   */
  public String toString()
  {
    String strLinLineStatus = "";
    
    strLinLineStatus += "OpMode: 0x" + ZeroHex.toZeroHexString(m_bOpMode) + ", ";
    strLinLineStatus += "Bitrate: " + m_wBitrate + " [BAUD], ";
    strLinLineStatus += "Flags: ";
   
    if(m_fRxOverrun)
      strLinLineStatus += "C";
    else
      strLinLineStatus += " ";
    
    if(m_fLinInitMode)
      strLinLineStatus += "I";
    else
      strLinLineStatus += " ";
    
    return strLinLineStatus;
  }

}
