/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.jni.*;
import de.ixxat.vci3.bal.FifoReader;

/**
 * This class implements the interface {@link ILinMessageReader} which
 * represents a LIN message reader.
 * 
 * @see ILinMessageReader
 * @author Michael Ummenhofer
 * 
 */
public class LinMessageReader extends FifoReader implements ILinMessageReader
{

  /**
   * constructor
   * @param pIFifoReader
   * @throws Throwable
   */
  protected LinMessageReader(CMalloc pIFifoReader) throws Throwable
  {
    super(pIFifoReader);
    //System.out.println("LinMessageReader::LinMessageReader");
    
  }

  /**
   * Clean up Object
   * @throws Throwable
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
 
  /**
   * @see ILinMessageReader#ReadMessage()
   * @return {@link LinMessage}
   * @throws Throwable
   */
  public LinMessage ReadMessage() throws Throwable
  {
    return ReadMessage(null);
  }
  
  /**
   * @see ILinMessageReader#ReadMessage(LinMessage)
   * @return {@link LinMessage}
   * @throws Throwable
   */
  public LinMessage ReadMessage(LinMessage oLinMsg) throws Throwable
  {
    LinMessage  oLinMsgLocal  = null;
    CPointer    pData         = null;
    
    synchronized (this)
    {
      try
      {
        // Lock access to FiFo
        super.Lock();

        // Try to get a LIN Message
        pData = super.AcquireRead();
        
        // Copy Message and Release FiFo Entry
        if(pData != null)
        {
          // Try to convert message
          try
          {
            // Create new LIN Message or update existing message
            if(oLinMsg == null)
            {
              oLinMsgLocal = new LinMessage(pData);
            }
            else
            {
              oLinMsg.fromCPointer(pData);
              oLinMsgLocal = oLinMsg;
            }
          }
          finally
          {
            // Release one message form queue
            // any failed to convert messages, too
            super.ReleaseRead();
          }
        }
      }
      finally
      {
        // Unlock access to FiFo
        super.Unlock();
      }
    }
    
    return oLinMsgLocal;
  }

}
